@@ -11,7 +11,7 @@
  * spawn lots of kits as children.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <sys/capability.h>
 #include <sys/stat.h>
@@ -29,17 +29,17 @@
 #include <Poco/Thread.h>
 #include <Poco/Util/Application.h>
 
-#include "Common.hpp"
-#include "IoUtil.hpp"
+#include <Common.hpp>
+#include <IoUtil.hpp>
 #include "Kit.hpp"
-#include "Log.hpp"
-#include "Unit.hpp"
-#include "Util.hpp"
+#include <Log.hpp>
+#include <Unit.hpp>
+#include <Util.hpp>
 
-#include "common/FileUtil.hpp"
-#include "common/Seccomp.hpp"
-#include "common/SigUtil.hpp"
-#include "security.h"
+#include <common/FileUtil.hpp>
+#include <common/Seccomp.hpp>
+#include <common/SigUtil.hpp>
+#include <security.h>
 
 using Poco::Process;
 using Poco::Thread;
@@ -49,9 +49,11 @@ using Poco::Util::Application;
 
 #ifndef KIT_IN_PROCESS
 static bool NoCapsForKit = false;
+static bool NoSeccomp = false;
 #endif
 static bool DisplayVersion = false;
 static std::string UnitTestLibrary;
+static std::string LogLevel;
 static std::atomic<unsigned> ForkCounter(0);
 
 static std::map<Process::PID, std::string> childJails;
@@ -75,7 +77,7 @@ public:
     bool pollAndDispatch()
     {
         std::string message;
-        const auto ready = readLine(message, [](){ return TerminationFlag.load(); });
+        const int ready = readLine(message, [](){ return TerminationFlag.load(); });
         if (ready <= 0)
         {
             // Termination is done via SIGTERM, which breaks the wait.
@@ -100,7 +102,7 @@ public:
             std::vector<std::string> tokens = LOOLProtocol::tokenize(message);
             if (tokens.size() == 2 && tokens[0] == "spawn")
             {
-                const auto count = std::stoi(tokens[1]);
+                const int count = std::stoi(tokens[1]);
                 if (count > 0)
                 {
                     LOG_INF("Setting to spawn " << tokens[1] << " child" << (count == 1 ? "" : "ren") << " per request.");
@@ -259,7 +261,7 @@ static int createLibreOfficeKit(const std::string& childRoot,
 
         if (std::getenv("SLEEPKITFORDEBUGGER"))
         {
-            const auto delaySecs = std::stoul(std::getenv("SLEEPKITFORDEBUGGER"));
+            const size_t delaySecs = std::stoul(std::getenv("SLEEPKITFORDEBUGGER"));
             if (delaySecs > 0)
             {
                 std::cerr << "Sleeping " << delaySecs
@@ -270,9 +272,9 @@ static int createLibreOfficeKit(const std::string& childRoot,
         }
 
 #ifndef KIT_IN_PROCESS
-        lokit_main(childRoot, jailId, sysTemplate, loTemplate, loSubPath, NoCapsForKit, queryVersion, DisplayVersion);
+        lokit_main(childRoot, jailId, sysTemplate, loTemplate, loSubPath, NoCapsForKit, NoSeccomp, queryVersion, DisplayVersion);
 #else
-        lokit_main(childRoot, jailId, sysTemplate, loTemplate, loSubPath, true, queryVersion, DisplayVersion);
+        lokit_main(childRoot, jailId, sysTemplate, loTemplate, loSubPath, true, true, queryVersion, DisplayVersion);
 #endif
     }
     else
@@ -348,7 +350,7 @@ int main(int argc, char** argv)
 
     if (std::getenv("SLEEPFORDEBUGGER"))
     {
-        const auto delaySecs = std::stoul(std::getenv("SLEEPFORDEBUGGER"));
+        const size_t delaySecs = std::stoul(std::getenv("SLEEPFORDEBUGGER"));
         if (delaySecs > 0)
         {
             std::cerr << "Sleeping " << delaySecs
@@ -376,7 +378,12 @@ int main(int argc, char** argv)
         logProperties["path"] = std::string(logFilename);
     }
 
-    Log::initialize("frk", logLevel ? logLevel : "", logColor != nullptr, logToFile, logProperties);
+    Log::initialize("frk", "trace", logColor != nullptr, logToFile, logProperties);
+    LogLevel = logLevel ? logLevel : "trace";
+    if (LogLevel != "trace")
+    {
+        LOG_INF("Setting log-level to [trace] and delaying setting to requested [" << LogLevel << "].");
+    }
 
     std::string childRoot;
     std::string loSubPath;
@@ -440,7 +447,7 @@ int main(int argc, char** argv)
             std::vector<std::string> tokens = LOOLProtocol::tokenize(rlimits, ';');
             for (const std::string& cmdLimit : tokens)
             {
-                const auto pair = LOOLProtocol::split(cmdLimit, ':');
+                const std::pair<std::string, std::string> pair = LOOLProtocol::split(cmdLimit, ':');
                 std::vector<std::string> tokensLimit = { "setconfig", pair.first, pair.second };
                 if (!Rlimit::handleSetrlimitCommand(tokensLimit))
                 {
@@ -455,12 +462,20 @@ int main(int argc, char** argv)
             eq = std::strchr(cmd, '=');
             UnitTestLibrary = std::string(eq+1);
         }
-        // we are running in no-privilege mode - with no chroot etc.
+#endif
+        // we are running in a lower-privilege mode - with no chroot
         else if (std::strstr(cmd, "--nocaps") == cmd)
         {
+            LOG_ERR("Security: Running without the capability to enter a chroot jail is ill advised.");
             NoCapsForKit = true;
         }
-#endif
+
+        // we are running without seccomp protection
+        else if (std::strstr(cmd, "--noseccomp") == cmd)
+        {
+            LOG_ERR("Security :Running without the ability to filter system calls is ill advised.");
+            NoSeccomp = true;
+        }
     }
 
     if (loSubPath.empty() || sysTemplate.empty() ||
@@ -478,7 +493,7 @@ int main(int argc, char** argv)
     }
 
     // Setup & check environment
-    std::string layers(
+    const std::string layers(
         "xcsxcu:${BRAND_BASE_DIR}/share/registry "
         "res:${BRAND_BASE_DIR}/share/registry "
         "bundledext:${${BRAND_BASE_DIR}/program/lounorc:BUNDLED_EXTENSIONS_USER}/registry/com.sun.star.comp.deployment.configuration.PackageRegistryBackend/configmgr.ini "
@@ -512,10 +527,14 @@ int main(int argc, char** argv)
     if (!globalPreinit(loTemplate))
         std::_Exit(Application::EXIT_SOFTWARE);
 
+    if (Util::getProcessThreadCount() != 1)
+        LOG_ERR("Error: forkit has more than a single thread after pre-init");
+
     LOG_INF("Preinit stage OK.");
 
     // We must have at least one child, more are created dynamically.
-    // Ask this first child to send version information to master process
+    // Ask this first child to send version information to master process and trace startup.
+    ::setenv("LOOL_TRACE_STARTUP", "1", 1);
     Process::PID forKitPid = createLibreOfficeKit(childRoot, sysTemplate, loTemplate, loSubPath, true);
     if (forKitPid < 0)
     {
@@ -523,6 +542,14 @@ int main(int argc, char** argv)
         std::_Exit(Application::EXIT_SOFTWARE);
     }
 
+    // No need to trace subsequent children.
+    ::unsetenv("LOOL_TRACE_STARTUP");
+    if (LogLevel != "trace")
+    {
+        LOG_INF("Setting log-level to [" << LogLevel << "].");
+        Log::logger().setLevel(LogLevel);
+    }
+
     CommandDispatcher commandDispatcher(0);
     LOG_INF("ForKit process is ready.");
 
