@@ -7,7 +7,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "ClientSession.hpp"
 
@@ -19,12 +19,12 @@
 
 #include "DocumentBroker.hpp"
 #include "LOOLWSD.hpp"
-#include "common/Common.hpp"
-#include "common/Log.hpp"
-#include "common/Protocol.hpp"
-#include "common/Session.hpp"
-#include "common/Unit.hpp"
-#include "common/Util.hpp"
+#include <common/Common.hpp>
+#include <common/Log.hpp>
+#include <common/Protocol.hpp>
+#include <common/Session.hpp>
+#include <common/Unit.hpp>
+#include <common/Util.hpp>
 
 using namespace LOOLProtocol;
 
@@ -68,7 +68,7 @@ bool ClientSession::_handleInput(const char *buffer, int length)
     const std::string firstLine = getFirstLine(buffer, length);
     const std::vector<std::string> tokens = LOOLProtocol::tokenize(firstLine.data(), firstLine.size());
 
-    auto docBroker = getDocumentBroker();
+    std::shared_ptr<DocumentBroker> docBroker = getDocumentBroker();
     if (!docBroker)
     {
         LOG_ERR("No DocBroker found. Terminating session " << getName());
@@ -91,7 +91,7 @@ bool ClientSession::_handleInput(const char *buffer, int length)
             return false;
         }
 
-        const auto versionTuple = ParseVersion(tokens[1]);
+        const std::tuple<int, int, std::string> versionTuple = ParseVersion(tokens[1]);
         if (std::get<0>(versionTuple) != ProtocolMajorVersionNumber ||
             std::get<1>(versionTuple) != ProtocolMinorVersionNumber)
         {
@@ -126,15 +126,20 @@ bool ClientSession::_handleInput(const char *buffer, int length)
     else if (tokens[0] != "canceltiles" &&
              tokens[0] != "clientzoom" &&
              tokens[0] != "clientvisiblearea" &&
+             tokens[0] != "outlinestate" &&
              tokens[0] != "commandvalues" &&
              tokens[0] != "closedocument" &&
+             tokens[0] != "versionrestore" &&
              tokens[0] != "downloadas" &&
              tokens[0] != "getchildid" &&
              tokens[0] != "gettextselection" &&
              tokens[0] != "paste" &&
              tokens[0] != "insertfile" &&
              tokens[0] != "key" &&
+             tokens[0] != "textinput" &&
+             tokens[0] != "windowkey" &&
              tokens[0] != "mouse" &&
+             tokens[0] != "windowmouse" &&
              tokens[0] != "partpagerectangles" &&
              tokens[0] != "ping" &&
              tokens[0] != "renderfont" &&
@@ -152,7 +157,9 @@ bool ClientSession::_handleInput(const char *buffer, int length)
              tokens[0] != "tilecombine" &&
              tokens[0] != "uno" &&
              tokens[0] != "useractive" &&
-             tokens[0] != "userinactive")
+             tokens[0] != "userinactive" &&
+             tokens[0] != "paintwindow" &&
+             tokens[0] != "windowcommand")
     {
         sendTextFrame("error: cmd=" + tokens[0] + " kind=unknown");
         return false;
@@ -185,11 +192,18 @@ bool ClientSession::_handleInput(const char *buffer, int length)
             LOG_DBG("Document marked as changed in storage and user ["
                     << getUserId() << ", " << getUserName()
                     << "] wants to refresh the document for all.");
-            docBroker->closeDocument("documentconflict " + getUserName());
+            docBroker->stop("documentconflict " + getUserName());
         }
 
         return true;
     }
+    else if (tokens[0] == "versionrestore") {
+        if (tokens[1] == "prerestore") {
+            // green signal to WOPI host to restore the version *after* saving
+            // any unsaved changes, if any, to the storage
+            docBroker->closeDocument("versionrestore: prerestore_ack");
+        }
+    }
     else if (tokens[0] == "partpagerectangles")
     {
         // We don't support partpagerectangles any more, will be removed in the
@@ -223,11 +237,12 @@ bool ClientSession::_handleInput(const char *buffer, int length)
     else if (tokens[0] == "save")
     {
         int dontTerminateEdit = 1;
-        int dontSaveIfUnmodified = 1;
         if (tokens.size() > 1)
             getTokenInteger(tokens[1], "dontTerminateEdit", dontTerminateEdit);
 
-        if (tokens.size() > 2)
+        // Don't save unmodified docs by default, or when read-only.
+        int dontSaveIfUnmodified = 1;
+        if (!isReadOnly() && tokens.size() > 2)
             getTokenInteger(tokens[2], "dontSaveIfUnmodified", dontSaveIfUnmodified);
 
         docBroker->sendUnoSave(getId(), dontTerminateEdit != 0, dontSaveIfUnmodified != 0);
@@ -403,7 +418,7 @@ bool ClientSession::sendTile(const char * /*buffer*/, int /*length*/, const std:
 {
     try
     {
-        auto tileDesc = TileDesc::parse(tokens);
+        TileDesc tileDesc = TileDesc::parse(tokens);
         docBroker->handleTileRequest(tileDesc, shared_from_this());
     }
     catch (const std::exception& exc)
@@ -420,7 +435,7 @@ bool ClientSession::sendCombinedTiles(const char* /*buffer*/, int /*length*/, co
 {
     try
     {
-        auto tileCombined = TileCombined::parse(tokens);
+        TileCombined tileCombined = TileCombined::parse(tokens);
         docBroker->handleTileCombinedRequest(tileCombined, shared_from_this());
     }
     catch (const std::exception& exc)
@@ -547,7 +562,7 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
     LOG_TRC(getName() << ": handling kit-to-client [" << payload->abbr() << "].");
     const std::string& firstLine = payload->firstLine();
 
-    const auto docBroker = _docBroker.lock();
+    const std::shared_ptr<DocumentBroker> docBroker = _docBroker.lock();
     if (!docBroker)
     {
         LOG_ERR("No DocBroker to handle kit-to-client message: " << firstLine);
@@ -561,12 +576,12 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
     {
         const std::string stringMsg(buffer, length);
         LOG_INF(getName() << ": Command: " << stringMsg);
-        const auto index = stringMsg.find_first_of('{');
+        const size_t index = stringMsg.find_first_of('{');
         if (index != std::string::npos)
         {
             const std::string stringJSON = stringMsg.substr(index);
             Poco::JSON::Parser parser;
-            const auto parsedJSON = parser.parse(stringJSON);
+            const Poco::Dynamic::Var parsedJSON = parser.parse(stringJSON);
             const auto& object = parsedJSON.extract<Poco::JSON::Object::Ptr>();
             if (object->get("commandName").toString() == ".uno:Save")
             {
@@ -574,15 +589,14 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
                 std::string result;
                 if (object->has("result"))
                 {
-                    const auto parsedResultJSON = object->get("result");
+                    const Poco::Dynamic::Var parsedResultJSON = object->get("result");
                     const auto& resultObj = parsedResultJSON.extract<Poco::JSON::Object::Ptr>();
                     if (resultObj->get("type").toString() == "string")
                         result = resultObj->get("value").toString();
                 }
 
                 // Save to Storage and log result.
-                std::string id = getId();
-                docBroker->saveToStorage(id, success, result);
+                docBroker->saveToStorage(getId(), success, result);
 
                 if (!isCloseFrame())
                     forwardToClient(payload);
@@ -709,11 +723,10 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
             LOG_TRC("Removing save-as ClientSession after conversion.");
 
             // Remove us.
-            std::string id = getId();
-            docBroker->removeSession(id);
+            docBroker->removeSession(getId());
 
             // Now terminate.
-            docBroker->stop();
+            docBroker->stop("Finished saveas handler.");
         }
 
         return true;
@@ -744,12 +757,12 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
         else if (tokens[0] == "commandvalues:")
         {
             const std::string stringMsg(buffer, length);
-            const auto index = stringMsg.find_first_of('{');
+            const size_t index = stringMsg.find_first_of('{');
             if (index != std::string::npos)
             {
                 const std::string stringJSON = stringMsg.substr(index);
                 Poco::JSON::Parser parser;
-                const auto result = parser.parse(stringJSON);
+                const Poco::Dynamic::Var result = parser.parse(stringJSON);
                 const auto& object = result.extract<Poco::JSON::Object::Ptr>();
                 const std::string commandName = object->has("commandName") ? object->get("commandName").toString() : "";
                 if (commandName == ".uno:CharFontName" ||
@@ -768,16 +781,23 @@ bool ClientSession::handleKitToClientMessage(const char* buffer, const int lengt
         else if (tokens[0] == "invalidatecursor:")
         {
             assert(firstLine.size() == static_cast<std::string::size_type>(length));
-            StringTokenizer firstLineTokens(firstLine, " ", StringTokenizer::TOK_IGNORE_EMPTY | StringTokenizer::TOK_TRIM);
+
+            const size_t index = firstLine.find_first_of('{');
+            const std::string stringJSON = firstLine.substr(index);
+            Poco::JSON::Parser parser;
+            const Poco::Dynamic::Var result = parser.parse(stringJSON);
+            const auto& object = result.extract<Poco::JSON::Object::Ptr>();
+            const std::string rectangle = object->get("rectangle").toString();
+            StringTokenizer rectangleTokens(rectangle, ",", StringTokenizer::TOK_IGNORE_EMPTY | StringTokenizer::TOK_TRIM);
             int x = 0, y = 0, w = 0, h = 0;
-            if (firstLineTokens.count() > 2 &&
-                stringToInteger(firstLineTokens[1], x) &&
-                stringToInteger(firstLineTokens[2], y))
+            if (rectangleTokens.count() > 2 &&
+                stringToInteger(rectangleTokens[0], x) &&
+                stringToInteger(rectangleTokens[1], y))
             {
-                if (firstLineTokens.count() > 3)
+                if (rectangleTokens.count() > 3)
                 {
-                    stringToInteger(firstLineTokens[3], w);
-                    stringToInteger(firstLineTokens[4], h);
+                    stringToInteger(rectangleTokens[2], w);
+                    stringToInteger(rectangleTokens[3], h);
                 }
 
                 docBroker->invalidateCursor(x, y, w, h);
@@ -859,20 +879,17 @@ void ClientSession::onDisconnect()
 {
     LOG_INF(getName() << " Disconnected, current number of connections: " << LOOLWSD::NumConnections);
 
-    const auto docBroker = getDocumentBroker();
+    const std::shared_ptr<DocumentBroker> docBroker = getDocumentBroker();
     LOG_CHECK_RET(docBroker && "Null DocumentBroker instance", );
     docBroker->assertCorrectThread();
-    const auto docKey = docBroker->getDocKey();
+    const std::string docKey = docBroker->getDocKey();
 
     try
     {
         // Connection terminated. Destroy session.
         LOG_DBG(getName() << " on docKey [" << docKey << "] terminated. Cleaning up.");
 
-        // We issue a force-save when last editable (non-readonly) session is going away
-        // and defer destroying the last session and the docBroker.
-        std::string id = getId();
-        docBroker->removeSession(id, true);
+        docBroker->removeSession(getId());
     }
     catch (const UnauthorizedRequestException& exc)
     {
